# attack.py
#
# Kevin Angstadt
# angstadt @ virginia.edu
# University of Virginia
#
# A script for "protecting" a sphere around a given GPS coordinate.
# When the MAV enters the protected zone, it is guided back out of
# the region while being instructed to continue to the next waypoint.

from pymavlink import mavutil,mavextra
from collections import namedtuple
from math import *
import argparse
import time

FakeGPS = namedtuple( 'FakeGPS', ['lat','lon','cog','alt'] )

# Helper function to determine the bearing between two GPS coordinates
def bearing( GPS_RAW1, GPS_RAW2 ):
    '''bearing of GPS_RAW2 relative to GPS_RAW1'''
    lat1 = radians( GPS_RAW1.lat ) * 1.0e-7
    lat2 = radians( GPS_RAW2.lat ) * 1.0e-7
    lon1 = radians( GPS_RAW1.lon ) * 1.0e-7
    lon2 = radians( GPS_RAW2.lon ) * 1.0e-7
    
    x = sin( lon2 - lon1 ) * cos( lat2 )
    y = cos( lat1 ) * sin( lat2 ) - sin( lat1 ) * cos( lat2 )*  cos( lon2 - lon1 )
    
    return degrees( atan2( x, y ) )

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument( "--baudrate",
                        type=int,
                        help="master port baud rate",
                        default=57600
                        )

    parser.add_argument( "--device",
                        required=True,
                        help="serial device"
                        )
    
    parser.add_argument( "--source-system",
                        dest='SOURCE_SYSTEM',
                        type=int,
                        default=255,
                        help='MAVLink source system for this GCS')
    
    parser.add_argument( "lat", type=float, help="latitude of point to protect" )
    parser.add_argument( "lon", type=float, help="longitude of point ot protect" )
    parser.add_argument( "radius", type=float, help="radius around point to protect" )
    
    args = parser.parse_args()
    
    # connect to mav
    master = mavutil.mavlink_connection( args.device,
                                        baud = args.baudrate,
                                        source_system=args.SOURCE_SYSTEM
                                        )
    
    # make up a fake gps coordinate to calculations
    protect = FakeGPS(args.lat*1.0e7,args.lon*1.0e7,0,10)
    
    restore_mode = None
    triggered = False

    # start protecting
    while True:
        # get the MAV's current position
        loc = master.recv_match( type='GPS_RAW_INT', blocking=True )
        
        # calculate distance and bearing from protected point
        dist = mavextra.distance_two( loc, protect )
        deg = mavextra.wrap_360( bearing( protect, loc ) )
        
        print dist
        
        
        # if we are in the 'cone of influence'
        if dist <= args.radius and master.motors_armed():
            print "Incoming target at %.2f degrees and %.2f meters away." % ( deg, dist )
            # we need to protect the point!
            
            # but first back up current mode
            if restore_mode is None:
                master.recv_match(type='SYS_STATUS')
                restore_mode = master.flightmode
                # set mode for brake
                master.set_mode(17)
                
                # if the attack hasn't occurred yet
                if not triggered:
                    # increment the current mission item
                    m_current = master.recv_match(type='MISSION_CURRENT',blocking=True)
                    seq = m_current.seq
                    
                    master.mav.mission_set_current_send(
                        0,
                        0,
                        seq+1
                    )
                    
                    triggered = True
                
                # We sleep for a moment to help the MAV slow down
                time.sleep(1)
                print "go"
                
            
            # find a new location to fly to
            lat,lon = mavextra.gps_newpos( loc.lat*1.0e-7,
                                           loc.lon*1.0e-7,
                                           deg,
                                           abs(args.radius - dist)
                                          )
            
            
            # set mode to guided
            master.set_mode('GUIDED')
            
            # send the target location
            master.mav.set_position_target_global_int_send(
                0, #timestamp
                0, #target_system
                0, #target_component
                6, # MAV_FRAME_GLOBAL_RELATIVE_ALT_INT
                0b0000111111111000, # I think this does only positions
                int(lat*1.0e7), #lat_int
                int(lon*1.0e7), #lon_int
                master.recv_match( type='GLOBAL_POSITION_INT', blocking=True ).relative_alt * 0.001, #alt
                0, # x velocity
                0, # y velocity
                0, # z velocity
                0,0,0, # accel not supported
                0,0 # yaw not supported
            )
        elif restore_mode is not None:
            # We are finished
            print 'Done protecting'
            master.set_mode(restore_mode)
            restore_mode = None


if __name__ == '__main__':
    main()